import java.io.*;
import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;
import org.apache.tools.ant.*;

/**
 * A simple listener that waits for a build to finish and sends an email
 * of the results.  To control the build, you must set a variety of
 * mail.* properties. See svn-build.properties for some examples.
 * One property that isn't mentioned there, but that is available, is
 * mail.translate to translate a user name into an email address.
 *
 * @author      $Author$
 */
public class BuildMonitor implements BuildListener {
    protected Hashtable table = null;
    protected Properties props = null;
    protected Project proj = null;
    protected Address[] recipientArray = null;
    protected Address[] replyToArray = null;
    protected Throwable th = null;
    private static String keyroot = "mail.";
    private String key;

    /**
     * Create a new BuildMonitor.
     */
    public BuildMonitor() throws Exception {
        props = new Properties();
    }

    public void buildStarted(BuildEvent e) {
    }

    /**
     * Determine the status of the build and the actions to follow, now that
     * the build has completed.
     *
     * @param       e       Event describing the build tatus.
     */
    public void buildFinished(BuildEvent e) {
        th = e.getException();
        proj = e.getProject();
        table = proj.getProperties();  //!!!
        getRealProperties();
        String status = (th != null) ? "failure." : "success.";

        try {
            keyroot = "mail.";
            key = keyroot + status;
            if (props.getProperty(key + "notify").equalsIgnoreCase("false")) {
                return;
            }

            Session session = Session.getDefaultInstance(props, null);

            MimeMessage message = new MimeMessage(session);
            String tostr = props.getProperty(key + "list");
            recipientArray = parseAddresses(tostr);
            message.addRecipients(Message.RecipientType.TO, recipientArray);
            String subject = props.getProperty(key + "subject");
            if (subject != null && subject.length() > 0) {
                message.setSubject(subject);
            }
            String replytostr = props.getProperty(key + "replyto");
            if (replytostr != null && replytostr.length() > 0) {
                replyToArray = parseAddresses(replytostr);
                message.setReplyTo(replyToArray);
            }

            StringWriter sw = new StringWriter();
            String header = props.getProperty(key + "header");
            sw.write(header);
            sw.write("\n");

            String loginclude = props.getProperty(key + "sendlog");
            if (loginclude != null && loginclude.length() > 0) {
                BufferedReader br = new BufferedReader(new FileReader(
                    props.getProperty(key + "logfilename")));

                String line = br.readLine();
                while (line != null) {
                    sw.write(line);
                    sw.write("\n");
                    line = br.readLine();
                }
                br.close();
            }

            message.setText(sw.toString(), "UTF-8");
            sw.close();

            Transport transport = session.getTransport();
            transport.connect();
            transport.send(message);
            transport.close();
        } catch (Exception ex) {
            System.out.println("BuildMonitor failed to send email!");
            ex.printStackTrace();
        }
    }

    /**
     * Parse a comma separated list of internet email addresses.
     *
     * @param       s       The list of addresses.
     */
    protected Address[] parseAddresses(String s) throws AddressException {
        Vector addrVec = new Vector();
        StringTokenizer st = new StringTokenizer(s, ",");

        int cnt = st.countTokens();

        for (int i = 0; i < cnt; i++) {
            String addr = st.nextToken();
            String fq_addr = props.getProperty(keyroot + "translate." + addr);
            if (fq_addr == null)
            {
                fq_addr = addr;
            }
            addrVec.add(new InternetAddress(fq_addr));
        }
        Address[] results = new Address[addrVec.size()];
        addrVec.copyInto(results);
        return results;
    }


    public void messageLogged(BuildEvent e) {
    }

    public void targetStarted(BuildEvent e) {
    }

    public void targetFinished(BuildEvent e) {
    }

    public void taskStarted(BuildEvent e) {
    }

    public void taskFinished(BuildEvent e) {
    }

    // Who knew that the difference between Properties and Hashtable
    // could cause such hell, given that one is a subclass of the other!
    // But project.getProperties returns a Hashtable, many calls demand
    // an actual Properties object, and there is no conversion constructor
    // from one to the other. So we have to do our own conversion.
    private void getRealProperties() {
        for (Enumeration coll = table.keys() ; coll.hasMoreElements() ;) {
            String s = (coll.nextElement()).toString();
            props.setProperty(s,(table.get(s)).toString());
        }
    }

}


